# -*- coding: utf-8 -*-
"""
Created on Fri Feb 10 11:12:33 2017

@author: Gereon Goldbeck, Armin Fohler
"""

# Write XML File for material components
#
# IMPORTANT NOTE:
# works with bezier version 0.9.0 up to 2020.2.3
# install by using the commend:
# 'pip install --upgrade bezier==0.9.0'
# or for the current version:
# 'pip install --upgrade bezier'

import xml.etree.cElementTree as etree
import numpy as np
from math import atan, sin, exp, floor
from scipy.optimize import curve_fit
import matplotlib.pyplot as plt
from matplotlib.font_manager import FontProperties
import bezier  # pip install --upgrade bezier
import scipy.io as sio
import copy
import os
from PyQt5 import QtCore
import threading


class ExtendBHCurve:

    @staticmethod
    def extend_bhcurve_froehlich_kennelly(material_dictionary, H, B, show_plots=True):
    
        mu_0 = 4 * np.pi * 1e-7
        b = (H[-1] / (B[-1] - H[-1] * mu_0) - H[-2] / (B[-2] - H[-2] * mu_0))/(H[-1] - H[-2])
        a = H[-1] / (B[-1] - H[-1] * mu_0) - b * H[-1]
        saturation_polarization = 1.0/b

        # step_size = 10000  # original = 1000
        # h_ext = np.arange(H[-1], 500000, step_size).tolist()[1:-1]
        
        h_ext = []
        step = H[-1]*1.2
        while step < 500000:
            h_ext.append(step)
            step = step * 1.2
        
        H_extrapolated = np.concatenate((H, h_ext), axis=None)
        
        b_ext = []
        for element in h_ext:
            b_ext.append(element / (a + b * element) + element * mu_0)
            
        B_extrapolated = np.concatenate((B, b_ext), axis=None)
        
        return H_extrapolated, B_extrapolated

    @staticmethod
    def extend_bhcurve(material_dictionary, type, H, B, show_plots=True):
        # # mu = 1/alpha+beta*|H|
        mu_0 = 4 * np.pi * 1e-7
        # H = material_dictionary['Magnetic field strength']['value'][0]
        # B = [a + b for a, b in zip([j / 1000.0 for j in material_dictionary['Magnetic polarization']['value'][0]],
        #                            [h * mu_0 for h in material_dictionary['Magnetic field strength']['value'][0]])]
        # if H[0] > 0.0:
        #     H.insert(0, 0.0)
        #     B.insert(0, 0.0)
        #
        # material_dictionary['H_original'] = [H]
        # material_dictionary['B_original'] = [B]

        H_extrapolated, h_ext = ExtendBHCurve.get_h_extension(H)
        # print('H extrapolated = ', H_extrapolated)

        mu_rel = np.array([(B[i] - B[i - 1]) / (H[i] - H[i - 1]) / mu_0 for i in range(1, len(H))])
        # mu_rel = np.append(mu_rel[0], mu_rel)
        # mur = 1 + (mu ) / (1 + beta * B + pow(B,n))

        if show_plots:
            plt.plot(B[0:-1], mu_rel, "bo")
            plt.title('B - mur_relativ')
            plt.xlabel('B')
            plt.ylabel('mur_rel')
            plt.show()

            plt.plot(H[0:-1], mu_rel, "go")
            plt.title('H - mur_relativ')
            plt.xlabel('H')
            plt.ylabel('mur_rel')
            plt.show()

        def fitting_function(x, a, b, c):
            y_value = []
            if type == 'arctan':
                # y_value = [a * atan(z * b + c) + np.pi / 2.0 * a + 1 for z in x]
                y_value = [max(a * atan(z * b + c) - a * atan(500000 * b + c) + 1, 1.0) for z in x]
            elif type == 'exp':
                y_value = [a * exp(z * b + c) + 1 for z in x]
            elif type == 'logistic':
                y_value = [1.0 / (1 + exp(-z * b) * (1.0 / c - 1)) for z in x]

            return y_value

        param_bounds = []
        if type == 'arctan':
            param_bounds = ([-10000, 0, -100], [-100, 1, 0])
        elif type == 'exp':
            param_bounds = (
            [0, -np.inf, -100], [np.inf, 0, 100])  # ([-np.inf, -np.inf, -np.inf], [np.inf, np.inf, np.inf])  #
        elif type == 'logistic':
            param_bounds = ([0, 0, 1000], [1, 1e-5, 20000])

        # fit_params, fit_covariance = curve_fit(fitting_function, H[-13:-1], mu_rel[-13:-1], bounds=param_bounds)
        fit_params, fit_covariance = curve_fit(fitting_function, H[0:-1], mu_rel, bounds=param_bounds)

        print(fit_params)
        # params_arctan = [0.1, 10]

        #mu_r_arctan = mu_rel
        #mu_r_arctan = np.concatenate((mu_r_arctan, fitting_function(h_ext, fit_params[0], fit_params[1], fit_params[2])), axis=None)
        mu_r_arctan = fitting_function(H_extrapolated, fit_params[0], fit_params[1], fit_params[2])
        B_arctan = [0]
        for i in range(len(H_extrapolated)-1):
            B_arctan.append(mu_r_arctan[i]*mu_0*(H_extrapolated[i+1]-H_extrapolated[i]) + B_arctan[i])

        if show_plots:
            plt.plot(H[0:-1], mu_rel, "ro", H_extrapolated[0:len(H) + 5], mu_r_arctan[0:len(H) + 5], "b-")
            plt.title('H - mur_relativ')
            plt.xlabel('H')
            plt.ylabel('mur_rel')
            plt.show()

        if show_plots:
            plt.plot(material_dictionary["H_original"], material_dictionary["B_original"], "ro", H_extrapolated, B_arctan, "b-")
            plt.title('BH-Curve')
            plt.xlabel('H')
            plt.ylabel('B')
            plt.show()

        if show_plots:
            plt.plot(H, B, "ro", H_extrapolated, B_arctan, "b-")
            plt.title('BH-Curve')
            plt.xlabel('H')
            plt.ylabel('B')
            plt.show()

        x_arctan = np.linspace(0, H_extrapolated[-1], floor(H_extrapolated[-1]/1000))
        y_arctan = fitting_function(x_arctan, fit_params[0], fit_params[1], fit_params[2])

        if show_plots:
            # plt.plot(H[-13:-1], mu_rel[-13:-1], "ro", H[0:-1], mu_rel, "b-")
            plt.plot(H[0:-1], mu_rel, "ro", H[0:-1], mu_rel, "b-")
            plt.plot(x_arctan[0:10000], y_arctan[0:10000], "g-")
            # plt.plot(x_arctan, y_arctan, "g-")
            plt.title('H - mur_relativ')
            plt.xlabel('H')
            plt.ylabel('mur_rel')
            plt.show()

        print('max_value = ', fitting_function([H_extrapolated[-1]], fit_params[0], fit_params[1], fit_params[2]))
        print('inf_value = ', fitting_function([1e10], fit_params[0], fit_params[1], fit_params[2]))

        if show_plots:
            plt.plot(H_extrapolated, mu_r_arctan, "ro", H[0:-1], mu_rel, "b-")
            plt.plot(x_arctan, y_arctan, "g-")
            plt.title('H - mur_relativ')
            plt.xlabel('H')
            plt.ylabel('mur_rel')
            plt.show()

        B_extrapolated = B
        n = len(mu_rel)
        for i in range(len(h_ext)):
            num = h_ext[i]
            b_value = mu_r_arctan[n + i] * mu_0 * (H_extrapolated[n + i + 1] - H_extrapolated[n + i]) + B_extrapolated[
                n + i]
            B_extrapolated = np.concatenate((B_extrapolated, b_value), axis=None)

        if show_plots:
            plt.plot(H, B, "ro", H_extrapolated[0:len(H) + 5], B_extrapolated[0:len(H) + 5], "b-")
            plt.title('BH-Curve')
            plt.xlabel('H')
            plt.ylabel('B')
            plt.show()

        plt.plot(H_extrapolated, B_extrapolated, "ro", H, B, "b-")
        # x = np.linspace(0, H_extrapolated[-1], 2 * H_extrapolated[-1])
        # y = test_arctan(x, params_arctan[0], params_arctan[1], params_arctan[2])

        if show_plots:
            # plt.plot(x, y, "g-")
            plt.title('BH-Curve')
            plt.xlabel('H')
            plt.ylabel('B')
            plt.show()

        print('mu_rel', mu_rel)
        print('mu_r_arctan', mu_r_arctan)

        return H_extrapolated, B_extrapolated

    @staticmethod
    def get_h_extension(H):
        ## Simple version #############################################################
        step_size = 10000  # original = 1000
        h_ext = np.arange(H[-1], 500000, step_size).tolist()[1:-1]
        H_extrapolated = np.concatenate((H, h_ext), axis=None)
        ###############################################################################
        # def fitting_function_exp(x, a, b, c):
        #     # y_value = [(a * atan(z * b)) for z in x]
        #     y_value = [(a * exp(z * b + c)) for z in x]
        #     return y_value
        #
        # param_bounds = ([-np.inf, -100, -100], [np.inf, 100, 100])
        #
        # # fit_params, fit_covariance = curve_fit(fitting_function_exp, H[-13:-1], mu_rel[-13:-1], bounds=param_bounds)
        # x = range(len(H))
        # fit_params, fit_covariance = curve_fit(fitting_function_exp, x, H, bounds=param_bounds)
        #
        # x_new = range(floor(len(H)*1.2))
        # print(fit_params)
        # h_arctan = fitting_function_exp(x_new, fit_params[0], fit_params[1], fit_params[2])
        #
        # plt.plot(x, H, "ro", x_new, h_arctan, "b-")
        # plt.title('H-Curve')
        # plt.xlabel('range')
        # plt.ylabel('H')
        # plt.show()
        #
        # x_eval = len(H)
        # h_ext = fitting_function_exp([x_eval], fit_params[0], fit_params[1], fit_params[2])
        # while h_ext[-1] < 500000:
        #     x_eval = x_eval+1
        #     h_ext = np.concatenate((h_ext, fitting_function_exp([x_eval], fit_params[0], fit_params[1], fit_params[2])), axis=None)
        #
        # H_extrapolated = np.concatenate((H, h_ext), axis=None)
        ###############################################################################
        # extension_factor = 1.2
        # h_ext = [H[-1] * extension_factor]
        # while h_ext[-1] < 500000:
        #     h_ext.append(h_ext[-1] * extension_factor)
        #
        # H_extrapolated = np.concatenate((H, h_ext), axis=None)
        ###############################################################################

        return H_extrapolated, h_ext

    @staticmethod
    def extend_bhcurve_bezier(material_dictionary, show_plots=True):
        # mu = 1/alpha+beta*|H|
        mu_0 = 4 * np.pi * 1e-7
        H = material_dictionary['Magnetic field strength']['value'][0]
        B = material_dictionary['Magnetic polarization']['value'][0] / 1000.0 + mu_0 * material_dictionary['Magnetic field strength']['value'][0]

        if H[0] > 0.0:
            H.insert(0, 0.0)
            B.insert(0, 0.0)

        # h_ext = [500000]
        # H_extrapolated = np.concatenate((H, h_ext), axis=None)
        H_extrapolated = H
        B_extrapolated = B

        if show_plots:
            plt.plot(H_extrapolated, B_extrapolated, "ro", H, B, "r-")
            plt.show()

        return H_extrapolated, B_extrapolated


# class CalcBezierFunc(QtCore.QThread):
class CalcBezierFunc(threading.Thread):
    """ seperate thread for running the measurement tasks """

    def __init__(self, initial_curve, measured_curve, lambda_B, lambda_H, degree, H_knick, B_knick):
        # QtCore.QThread.__init__(self)
        print('bezier object')
        threading.Thread.__init__(self)
        self.curve_1 = initial_curve
        self.curve_2 = measured_curve
        self.lambda_B_values = lambda_B
        self.lambda_H_values = lambda_H
        self.degree = degree
        self.knick = [H_knick, B_knick]
        self.res_degree = 0
        self.err = np.Inf
        print('bezier thread done')

    # def __del__(self):
    #     self.wait()

    def run(self):
        # run is never called explicit, automatically called by Qt4 after start of thread
        # create parameters objects

        self.err = np.Inf

        try:
            print('run')
            # for degree in self.degrees:
            for lambda_H in self.lambda_H_values:
                print('lambda_H')
                for lambda_B in self.lambda_B_values:
                    print('lambda_B')
                    err_tmp = funcBezier(np.array(self.curve_1["B"]), np.array(self.curve_1["H"]), lambda_B, lambda_H,
                                         self.knick[1], self.knick[0], int(self.degree), self.curve_2)
                    # print(err_tmp)
                    print('self.err', self.err)
                    print('err_tmp', err_tmp)
                    if self.err > err_tmp:
                        # print(err_tmp)
                        self.err = copy.deepcopy(err_tmp)
                        self.res_lambda_B = lambda_B
                        self.res_lambda_H = lambda_H

            # return {"degree": self.res_degree, "lambda_B": self.lambda_B, "lambda_H": self.lambda_H, "error": self.err}

        except:
            print('run exception')
            # return {"degree": 0.0, "lambda_B": self.lambda_B, "lambda_H": self.lambda_H, "error": np.Inf}
            pass


def getInitualBHcurve(meas):
    # Erstellen der Neukurve aus den Messdaten als Dictionary:
    initial_curve = {"B": [0], "H": [0], "mu_r": [0], "f": meas[0].frequenz}
    for i in range(len(meas)):
        if (meas[i].hmax > 0.0) and (meas[i].bmax > 0.0):
            if (meas[i].hmax > initial_curve["H"][-1]):  # Hier werden Messfehler ausgeschlossen!
                initial_curve["B"].append(meas[i].bmax)
                initial_curve["H"].append(meas[i].hmax)
                initial_curve["mu_r"].append(meas[i].bmax / (meas[i].hmax * 4. * np.pi * 1e-7))
    # Der Punkt (0,0) ist wegen Division durch NULL nicht definiert, er muss gesetzt werden!:
    initial_curve["mu_r"][0] = initial_curve["mu_r"][1]
    return initial_curve


def calcInitualBHcurveConstEnergy(initial_curve):
    # Modify B(H)-Curve for FEM-Simulations:
    # Integral der Neukurve muss gleich sein!
    integ = 0.0
    i_stopped_loop = 0
    turning_point = False  # Ist der Wendepunkt erreicht?
    initial_curve_mod = {"B": [0], "H": [0], "mu_r": [0], "f": initial_curve["f"]}
    for i in range(len(initial_curve["B"]) - 2):
        deltaH = (initial_curve["H"][i + 1] - initial_curve["H"][i + 0])
        deltaB = (initial_curve["B"][i + 1] - initial_curve["B"][i + 0])
        dBdH1 = deltaB / deltaH  # Ableitung 1. Stück

        integ += (0.5 * deltaB * deltaH + initial_curve["B"][i] * deltaH)  # Integral der tatsächlichen Neukurve

        deltaH = (initial_curve["H"][i + 2] - initial_curve["H"][i + 1])
        deltaB = (initial_curve["B"][i + 2] - initial_curve["B"][i + 1])
        dBdH2 = deltaB / deltaH  # Ableitung 2. Stück

        integNewCurve = 0.5 * initial_curve["B"][i + 1] * initial_curve["H"][i + 1]  # Integral der angepassten Neukurve

        if (dBdH2 < dBdH1) and not turning_point:
            turning_point = True  # Der Wendepunkt ist überschritten/erreicht

        if turning_point:
            if (
                    integNewCurve < integ):  # Fehleranfällig, siehe Notizblock!  # or i == (len(initial_curve["B"]) - 4) !!!!!!!!!!!!!!!!
                # Sobald die Integrale nahezu wieder gleich werden,
                # kann der Rest der Neukurve angehängt werden
                for i2 in range(len(initial_curve["B"]) - i - 1):
                    initial_curve_mod["B"].append(initial_curve["B"][i2 + i + 1])
                    initial_curve_mod["H"].append(initial_curve["H"][i2 + i + 1])
                    initial_curve_mod["mu_r"].append(
                        initial_curve["B"][i2 + i + 1] / (initial_curve["H"][i2 + i + 1] * 4. * np.pi * 1e-7))
                initial_curve_mod["mu_r"][0] = initial_curve_mod["mu_r"][1]

                return initial_curve_mod
        i_stopped_loop = i
    # Falls nach dem letzten Wendepunkt kein Punkt mehr gefunden wird, wo die Energie/ das Integral zwischen angepasster
    # B(H)-Kurve und Messkurve wieder übereinstimmt, wird lediglich eine Gerade angefügt.
    # Möglich ist hier noch eine Anpassung der Punkte bzgl. konstanter Energie / gleichem Integral.
    if (initial_curve["H"][-1] > initial_curve["H"][-2]):
        last_index = -1
    else:
        last_index = -2
    initial_curve_mod["B"].append(initial_curve["B"][last_index])
    initial_curve_mod["H"].append(initial_curve["H"][last_index])
    initial_curve_mod["mu_r"].append(
        initial_curve["B"][last_index] / (initial_curve["H"][last_index] * 4. * np.pi * 1e-7))
    initial_curve_mod["mu_r"][0] = initial_curve_mod["mu_r"][1]

    return initial_curve_mod


def checkStrictlyDecreasing(initial_curve):
    for i in range(len(initial_curve["B"]) - 2):
        deltaH = (initial_curve["H"][i + 1] - initial_curve["H"][i + 0])
        deltaB = (initial_curve["B"][i + 1] - initial_curve["B"][i + 0])
        dBdH1 = deltaB / deltaH  # Ableitung 1. Stück

        # integ += (0.5 * deltaB * deltaH + initial_curve["B"][i] * deltaH)  # Integral der tatsächlichen Neukurve

        deltaH = (initial_curve["H"][i + 2] - initial_curve["H"][i + 1])
        deltaB = (initial_curve["B"][i + 2] - initial_curve["B"][i + 1])
        dBdH2 = deltaB / deltaH  # Ableitung 2. Stück

        # integNewCurve = 0.5 * initial_curve["B"][i + 1] * initial_curve["H"][i + 1]  # Integral der angepassten Neukurve

        if (dBdH2 > dBdH1):
            # print("Zweiter Wendepunkt gefunden!")
            H = np.array(initial_curve["H"][i:]) - np.array(initial_curve["H"][i])
            B = np.array(initial_curve["B"][i:]) - np.array(initial_curve["B"][i])
            if H[0] > 0.0:
                mu_r = B / (H * 4.0 * np.pi * 1e-7)
            else:
                mu_r = B[1:] / (H[1:] * 4.0 * np.pi * 1e-7)
                mu_r = np.append(mu_r[0], mu_r)
            segment_curve = {"B": B.tolist(), "H": H.tolist(), "mu_r": mu_r.tolist(), "f": initial_curve["f"]}
            segment_curve_mod = calcInitualBHcurveConstEnergy(segment_curve)

            H_segment = np.array(segment_curve_mod["H"][1:]) + np.array(initial_curve["H"][i])
            B_segment = np.array(segment_curve_mod["B"][1:]) + np.array(initial_curve["B"][i])
            # mu_r_segment = np.array(segment_curve_mod["mu_r"][1:]) + np.array(initial_curve["mu_r"][i])

            H_new = initial_curve["H"][:i + 1] + H_segment.tolist()
            B_new = initial_curve["B"][:i + 1] + B_segment.tolist()
            if H_new[0] > 0.0:
                mu_r_new = (np.array(B_new) / (np.array(
                    H_new) * 4.0 * np.pi * 1e-7)).tolist()  # initial_curve["mu_r"][i-1:] + segment_curve_mod["mu_r"][:]
                mu_r_new[0] = mu_r_new[1]
            else:
                mu_r_new = (np.array(B_new[1:]) / (np.array(H_new[1:]) * 4.0 * np.pi * 1e-7)).tolist()
                mu_r_new = np.append(mu_r_new[0], mu_r_new)

            return {"B": B_new, "H": H_new, "mu_r": mu_r_new.tolist(), "f": initial_curve["f"]}

    return initial_curve


def calcInitualBHcurve(initial_curve):
    # Modify B(H)-Curve for FEM-Simulations:
    # Nur Betrachtung der der Steigung (Vorraussetzung: monoton steigend)
    for _ in range(len(initial_curve["B"])):
        dBdH1 = initial_curve["B"][1] / initial_curve["H"][1]
        dBdH2 = (initial_curve["B"][2] - initial_curve["B"][1]) / (initial_curve["H"][2] - initial_curve["H"][1])
        if dBdH2 > dBdH1:
            initial_curve["B"].pop(1)
            initial_curve["H"].pop(1)
    return initial_curve


def polished(initial_curve):
    step_size = int(initial_curve["B"][1] / (initial_curve["B"][2] - initial_curve["B"][1]))
    inserted_list_B = (np.linspace(initial_curve["B"][0], initial_curve["B"][1], step_size + 1)[1:]).tolist()
    inserted_list_mu_r = [initial_curve["Mu"][0]] * len(inserted_list_B)

    mr_strAr = initial_curve["Mu"].tolist()
    b_strAr = initial_curve["B"].tolist()

    for i in range(step_size):
        mr_strAr.insert(i + 1, inserted_list_mu_r[i])
        b_strAr.insert(i + 1, inserted_list_B[i])

    B = np.array([])
    Mu = np.array([])

    samples = len(b_strAr)

    for i in range(samples):
        B = np.append(B, float(b_strAr[i]))
        Mu = np.append(Mu, float(mr_strAr[i]))
    H = B / Mu / (4 * np.pi * 1e-7)  # gemessene Daten

    B_part_1 = np.array([])
    Mu_part_1 = np.array([])

    B_mod = np.array([])
    H_mod = np.array([])
    Mu_mod = np.array([])

    turning_point = False  # Ist der Wendepunkt erreicht?

    for i in range(len(B) - 2):
        deltaH = B[i + 1] - B[i + 0]
        deltaMu = Mu[i + 1] - Mu[i + 0]
        dBdH1 = deltaMu / deltaH  # Ableitung 1. Stück

        deltaH = (B[i + 2] - B[i + 1])
        deltaMu = (Mu[i + 2] - Mu[i + 1])
        dBdH2 = deltaMu / deltaH  # Ableitung 2. Stück

        B_part_1 = np.append(B_part_1, B[i])
        Mu_part_1 = np.append(Mu_part_1, Mu[i])

        if (dBdH1 < dBdH2) and not turning_point:
            turning_point = True  # Der Wendepunkt ist überschritten/erreicht

            B_part_1 = np.append(B_part_1[:-1], (2 * B_part_1[-1] - B_part_1[::-1]))
            Mu_part_1 = np.append(Mu_part_1[:-1], (-(Mu_part_1 - 2 * Mu_part_1[-1])[::-1]))
            B_part_2 = B[i:]
            Mu_part_2 = Mu[i:]
            B_part_2 = np.append(B_part_2, 5)
            Mu_part_2 = np.append(Mu_part_2, 1.0)
            B_part_2 = np.append(2 * B_part_2[0] - B_part_2[::-1], (B_part_2[1:]))
            Mu_part_2 = np.append((-(Mu_part_2 - 2 * Mu_part_2[0])[::-1]), Mu_part_2[1:])

            break

    print(len(B_part_1))
    print(len(B_part_2))

    fig = plt.figure()
    plt.plot(B_part_2, Mu_part_2, 'b')
    plt.plot(B_part_1, Mu_part_1, 'r')

    ######################## PART 1:

    # B_part = B_part_1
    # Mu_part = Mu_part_1
    #
    # samples = len(B_part)
    # index_mitte_B_part = int(len(B_part) * 0.5)
    #
    # a1 = (np.abs(B_part) > B_part[index_mitte_B_part] - 0.3)
    # a2 = (np.abs(B_part) < B_part[index_mitte_B_part] + 0.3)
    # a3 = a1
    #
    # for i in range(samples):
    #     if a1[i] == True and a2[i] == True:
    #         a3[i] = True
    #     else:
    #         a3[i] = False
    #
    # sigmaB = np.ones(samples)
    # sigmaB[a3] = 0.01
    # sigmaB[index_mitte_B_part - 1:index_mitte_B_part + 1] = 0.001
    # sigmaB[-1] = 1
    # sigmaB[6:10] = 0.001
    #
    # # bound_part = ([0,-1000,-1000,-1000, 0. * np.min(Mu_part)], [1000000, 100000, 1000, 100000, 0.6 * np.max(Mu_part)])
    #
    # bound_part = ([0, 0, 0.8 * B_part[index_mitte_B_part], -1.2 * np.abs(np.min(Mu_part))],
    #               [1.2 * (np.max(Mu_part) - np.min(Mu_part)), 200, 1.2 * B_part[index_mitte_B_part],
    #                1.2 * np.abs(np.min(Mu_part))])
    # # bound_part = ([-400, 0, -0.6*B_part[index_mitte_B_part], 400, -11], [0, 4, 1.4*B_part[index_mitte_B_part], 600, -9])
    #
    # popt, pcov = curve_fit(func4, B_part, Mu_part, sigma=sigmaB, absolute_sigma=True, bounds=bound_part)
    # print(popt)
    #
    # B_test = np.linspace(0., B_part[index_mitte_B_part], 100)
    # Mu_test = func4(B_test, *popt)
    #
    # plt.plot(B_test, Mu_test, 'g')
    #
    # B_mod = np.append(B_mod, B_test)
    # Mu_mod = np.append(Mu_mod, Mu_test)

    ######################## PART 2:

    B_part = B_part_2
    Mu_part = Mu_part_2

    samples = len(B_part)
    index_mitte_B_part = int(len(B_part) * 0.5)

    a1 = (np.abs(B_part) > B_part[index_mitte_B_part] - 0.3)
    a2 = (np.abs(B_part) < B_part[index_mitte_B_part] + 0.3)
    a3 = a1

    for i in range(samples):
        if a1[i] == True and a2[i] == True:
            a3[i] = True
        else:
            a3[i] = False

    sigmaB = np.ones(samples)
    sigmaB[a3] = 0.005
    sigmaB[index_mitte_B_part - 1:index_mitte_B_part + 1] = 0.001
    sigmaB[1] = 0.01
    sigmaB[-1] = 0.01
    sigmaB[-2] = 0.001
    # bound_part = ([0,-1000,-1000,-1000, 0. * np.min(Mu_part)], [1000000, 100000, 1000, 100000, 0.6 * np.max(Mu_part)])

    bound_part = ([0.7 * Mu_part[index_mitte_B_part], 0.5 * np.abs(np.max(Mu_part) - np.min(Mu_part)), 0.001,
                   0.5 * B_part[index_mitte_B_part], -300],
                  [1.3 * Mu_part[index_mitte_B_part], 2 * np.max(Mu_part), 20000, 1.5 * B_part[index_mitte_B_part],
                   0.0])
    popt, pcov = curve_fit(func3, B_part, Mu_part, sigma=sigmaB, absolute_sigma=True, bounds=bound_part)
    print(popt)

    B_test = np.linspace(B_part[index_mitte_B_part], np.max(B_part), 100)
    Mu_test = func3(B_test, *popt)

    plt.plot(B_test, Mu_test, 'g')
    fig.show()

    # B_mod = np.append(B_mod[:], B[index_mitte_B_part + 1:])
    # Mu_mod = np.append(Mu_mod[:], Mu[index_mitte_B_part + 1:])
    # H_mod = B_mod / Mu_mod / (4. * np.pi * 1e-7)

    B_mod = np.append(B_mod, B_test)
    Mu_mod = np.append(Mu_mod, Mu_test)
    H_mod = B_mod / Mu_mod / (4. * np.pi * 1e-7)

    fig = plt.figure()
    plt.plot(H, B, 'b')  # gemessene Daten
    plt.plot(H_mod, B_mod, 'g')
    fig.show()

    #### Nur Extrapolation anfügen:

    fig = plt.figure()
    B_test = np.linspace(np.max(initial_curve["B"]) + 0.05, 2.5, 100)
    Mu_test = func3(B_test, *popt)
    plt.plot(initial_curve["B"], initial_curve["Mu"], 'b')
    plt.plot(B_test, Mu_test, 'g')
    fig.show()

    H_test = B_test / Mu_test / (4.0 * np.pi * 1e-7)

    fig = plt.figure()
    plt.plot(initial_curve["H"], initial_curve["B"], 'b')
    plt.plot(H_test, B_test, 'g')
    ax2 = fig.add_subplot(111)
    ax2.grid(True)
    ax2.set_xlabel("H [A/m]")
    ax2.set_ylabel("B [T]")
    ax2.set_xlim(0, max(initial_curve["H"]) * 8)
    ax2.set_ylim(0, 2.5)
    fig.show()

    return {"B": np.append(initial_curve["B"], B_test), "Mu": np.append(initial_curve["Mu"], Mu_test),
            "H": np.append(initial_curve["H"], H_test)}


def bezierPoints(p1, p2, p3):
    nodes = np.array([p1, p2, p3]).transpose()
    curve = bezier.Curve(nodes, degree=2)
    curve_start = 0.0
    curve_end = 1.0
    points_eval = curve.evaluate_multi(np.linspace(curve_start, curve_end, 20))
    # x = points_eval.transpose()[0]
    # y = points_eval.transpose()[1]
    return points_eval


def polishedByCircle(initial_curve, show_plots=True):
    min_inclusion_angle = 7.0
    offset = 0
    step_size = int(initial_curve["B"][1] / (initial_curve["B"][2] - initial_curve["B"][1]))
    inserted_list_B = (np.linspace(initial_curve["B"][0], initial_curve["B"][1], step_size + 1)[1:]).tolist()
    inserted_list_mu_r = [initial_curve["mu_r"][0]] * len(inserted_list_B)
    Mu_0 = (4.0 * np.pi * 1e-7)
    mr_strAr = initial_curve["mu_r"]
    b_strAr = initial_curve["B"]

    for i in range(step_size - 1):
        mr_strAr.insert(i + 1, inserted_list_mu_r[i])
        b_strAr.insert(i + 1, inserted_list_B[i])

    mr_strAr_inserted_point = []
    b_strAr_inserted_point = []
    for i in range(len(b_strAr) - 1):
        mr_strAr_inserted_point.append(mr_strAr[i])
        b_strAr_inserted_point.append(b_strAr[i])
        mr_strAr_inserted_point.append(0.5 * (mr_strAr[i] + mr_strAr[i + 1]))
        b_strAr_inserted_point.append(0.5 * (b_strAr[i] + b_strAr[i + 1]))
    mr_strAr_inserted_point.append(mr_strAr[-1])
    b_strAr_inserted_point.append(b_strAr[-1])

    # mr_strAr_inserted_point = mr_strAr
    # b_strAr_inserted_point = b_strAr

    B = np.array([])
    Mu = np.array([])
    # samples = len(b_strAr)
    samples = len(b_strAr_inserted_point)

    for i in range(samples):
        B = np.append(B, float(b_strAr_inserted_point[i]))
        Mu = np.append(Mu, float(mr_strAr_inserted_point[i]))
    H = B / Mu / Mu_0  # gemessene Daten

    for i in range(samples):
        # Sobald die µ(B)-Kurve abfällt:
        if np.abs(Mu[i + 1]) < 0.99 * np.abs(Mu[i]):
            indexBeginn = i - 1
            break

    # Werte auf 1 normieren; x:B(T), y:H(A/m)
    x_values = B / np.max(B)
    y_values = H / np.max(H)

    xy = np.array([x_values, y_values]).transpose()
    xy_Arr = xy.tolist()

    new_curve = xy_Arr[:indexBeginn]
    index = indexBeginn

    while True:

        # Betrachtung dreier Punkte und des Knickwinkels
        # Ab einer bestimmten Größe dieses Winkel, wird ein Kreisabschnitt zwischen den äußeren Punkten gelegt,
        # d.h. der erste der drei Punkte (p1) wird gedreht und
        # auf die Linie zwischen dem zweiten und dritten Punkt gelegt (p2)
        # Mit diesen zwei Punkten (p1 und p2) wird der kreisabschnitt definiert.
        p_0 = np.array(xy_Arr[offset + index - 1])
        p_1 = np.array(xy_Arr[offset + index])
        p_2 = np.array(xy_Arr[offset + index + 1])

        p1 = p_0  # xy[offset + index - 1]  #  (1 / 2) * xy[index + 0] + (1 / 2) * xy[index + 1]  #
        # p2 = xy[index + 2]  # (1 / 2) * xy[index + 2] + (1 / 2) * xy[index + 1]
        # p1 = p_0 + 0.1 * (p_0 - p_1)

        linie_01 = np.sqrt((p_1 - p_0)[0] ** 2 + (p_1 - p_0)[1] ** 2)  # Vektor zwischen 1. und 2. Punkt
        linie_02 = np.sqrt((p_2 - p_1)[0] ** 2 + (p_2 - p_1)[1] ** 2)  # Vektor zwischen 2. und 3. Punkt

        p2 = p_1 + ((p_2 - p_1) * linie_01 / linie_02)

        # Differenzvektoren in den drei betrachteten Punkten
        # Damit werden die Normalgeraden berechnet:
        diffPoint = p_1 - p1
        r_g1 = np.array([np.cos(0.5 * np.pi) * diffPoint[0] + np.sin(0.5 * np.pi) * diffPoint[1],
                         -np.sin(0.5 * np.pi) * diffPoint[0] + np.cos(0.5 * np.pi) * diffPoint[1]]) * -1.
        diffPoint = p_1 - p2
        r_g2 = np.array([np.cos(-0.5 * np.pi) * diffPoint[0] + np.sin(-0.5 * np.pi) * diffPoint[1],
                         -np.sin(-0.5 * np.pi) * diffPoint[0] + np.cos(-0.5 * np.pi) * diffPoint[1]]) * -1

        p1_p2 = p2 - p1

        lam_g2 = (p2[0] - p1[0] - (p2[1] - p1[1]) * r_g1[0] / r_g1[1]) / (r_g2[1] * r_g1[0] / r_g1[1] - r_g2[0])
        lam_g1 = (p2[1] + (lam_g2 * r_g2[1]) - p1[1]) / r_g1[1]

        # s_g1_g2_1 = p1 + lam_g1 * r_g1
        s_g1_g2_2 = p2 + lam_g2 * r_g2  # Schnittpunkt der Normal-Geraden
        s_g1_g2 = s_g1_g2_2
        p1_s = s_g1_g2 - p1

        abs_p1_p2 = np.sqrt(p1_p2[0] * p1_p2[0] + p1_p2[1] * p1_p2[1])  # Betrag des Vektors zwischen P1 und P2
        abs_p1_s = np.sqrt(p1_s[0] * p1_s[0] + p1_s[1] * p1_s[1])  # Betrag des Vektors zwischen P1 und S

        # Berechnung des eingeschlossenen Winkels der Normalgeraden:
        abs_r1 = np.sqrt(r_g1[0] ** 2 + r_g1[1] ** 2)
        abs_r2 = np.sqrt(r_g2[0] ** 2 + r_g2[1] ** 2)
        up_arg = r_g1[0] * r_g2[0] + r_g1[1] * r_g2[1]
        inclusion_angle = np.arccos(up_arg / (abs_r1 * abs_r2))
        # print(" r_g1 = " + str(r_g1) + " | r_g2 = " + str(r_g2))
        # print("  einschlusswinkel = " + str(inclusion_angle*180/np.pi))

        if inclusion_angle * 180 / np.pi > min_inclusion_angle:

            bpoints = (bezierPoints(p_0, p_1, p_2).tolist()).transpose()

            # xy_Arr.pop(offset + index - 1)
            # xy_Arr.pop(offset + index - 1)
            # xy_Arr.pop(offset + index - 1)

            for ins in range(len(bpoints) - 1):
                # xy_Arr.insert(offset + index - 1, bpoints[-ins-1])
                new_curve.append(bpoints[ins + 1])

            offset += 1
            # if lam_g1 > 0.0:
            #     rotation_direction = 1.0
            # else:
            #     rotation_direction = -1.0
            # rot_point = p1 - s_g1_g2
            # steps = int(inclusion_angle*180/np.pi)
            # new_curve.append(p_0.tolist())
            # new_curve.append(p1.tolist())
            # for i_step in range(steps):
            #     angle_step = i_step * rotation_direction * inclusion_angle / steps
            #     arc_point = np.array([np.cos(angle_step) * rot_point[0] - np.sin(angle_step) * rot_point[1],
            #                           np.sin(angle_step) * rot_point[0] + np.cos(angle_step) * rot_point[1]])
            #     new_curve.append((s_g1_g2 + arc_point).tolist())
            # while True:
            #     if xy_Arr[offset + index][0] > p2.tolist()[0]:
            #         xy_Arr.insert(offset + index, p2.tolist())
            #         xy_Arr.insert(offset + index, new_curve[-1])
            #         break
            #     else:
            #         # Falls beim Kreisbogeneinfügen der letzte Punkt weiter rechts liegt,
            #         # wirds der Punkt der aus der Urpsrungsliste gelöscht
            #         xy_Arr.pop(offset + index)
        else:
            new_curve.append(xy_Arr[offset + index])
        if (offset + index + 2) >= len(xy_Arr):
            for i in range((offset + index), len(xy_Arr)):
                new_curve.append(xy_Arr[i])
            break
        index += 1

    for ind in range(len(new_curve) - 1):
        try:
            if (new_curve[ind][0] == new_curve[ind + 1][0]):
                new_curve.pop(ind)
        except:
            break

    # Da die Rechnung auf normierte Werte basiert, müssen die Werte wieder skaliert werden!
    B_new = np.array(new_curve).transpose()[0] * np.max(B)
    H_new = np.array(new_curve).transpose()[1] * np.max(H)
    Mu_new = B_new / H_new / Mu_0
    Mu_new[0] = Mu_new[1]  # Der Punkt (0,0) ist wegen Division durch NULL nicht definiert, er muss gesetzt werden!

    initial_curve_smoothed = {"B": B_new, "mu_r": Mu_new, "H": H_new}

    #### PLOTS:

    # fig = plt.figure()
    # ax = fig.add_subplot(1, 1, 1)
    # ax.grid(True)
    # ax.set_xlabel("B [T]")
    # ax.set_ylabel("µ [1]")
    # ax.set_title("µ(B)")
    # plt.plot(B, Mu)
    # plt.plot(B, Mu, 'x')
    # plt.plot(B_new, Mu_new)
    # fig.show()
    #
    # fig = plt.figure()
    # ax = fig.add_subplot(1, 1, 1)
    # ax.grid(True)
    # ax.set_xlabel("H [A/m]")
    # ax.set_ylabel("B [T]")
    # ax.set_title("B(H)")
    # plt.plot(H, B)
    # plt.plot(H, B, 'x')
    # plt.plot(H_new, B_new)
    # fig.show()

    if show_plots:
        fontP = FontProperties()
        fontP.set_size('small')
        fig = plt.figure()
        ax = fig.add_subplot(211)
        ax.grid(True)
        ax.set_xlabel("B [T]")
        ax.set_ylabel("µ [1]")
        ax.set_xlim(0, max(B) * 1.1)
        ax.set_ylim(0, max(Mu) * 1.1)
        ax.set_title("Initial Curve for FE simulation")
        plt.plot(B, Mu)
        plt.plot(B, Mu, 'x')
        plt.plot(B_new, Mu_new)
        plt.plot(B_new, Mu_new, 'o')
        ax.legend(["measured curve", "measured points", "polished curve"], prop=fontP, loc='lower left')
        ax2 = fig.add_subplot(212)
        ax2.grid(True)
        ax2.set_xlabel("H [A/m]")
        ax2.set_ylabel("B [T]")
        ax2.set_xlim(0, max(H) * 1.1)
        ax2.set_ylim(0, max(B) * 1.1)
        plt.plot(H, B)
        plt.plot(H, B, 'x')
        plt.plot(H_new, B_new)
        ax2.legend(["measured curve", "measured points", "polished curve"], prop=fontP, loc='lower right')
        fig.show()

        fig = plt.figure()
        ax = fig.add_subplot(211)
        ax.grid(True)
        ax.set_xlabel("B [T]")
        ax.set_ylabel("µ [1]")
        ax.set_xlim(0, max(B) * 1.1)
        ax.set_ylim(0, max(Mu) * 1.1)
        ax.set_title("Initial Curve for FE simulation")
        plt.plot(B_new, Mu_new)
        ax.legend(["polished curve"], prop=fontP, loc='lower left')
        ax2 = fig.add_subplot(212)
        ax2.grid(True)
        ax2.set_xlabel("H [A/m]")
        ax2.set_ylabel("B [T]")
        ax2.set_xlim(0, max(H) * 1.1)
        ax2.set_ylim(0, max(B) * 1.1)
        plt.plot(H_new, B_new)
        ax2.legend(["polished curve"], prop=fontP, loc='lower right')
        fig.show()

    return initial_curve_smoothed


def showBezierDegree(p1, p2, p3, degree):
    print('showBezierDegree')
    array = [p1.tolist()] + (degree - 1) * [p2.tolist()] + [p3.tolist()]
    nodes = np.array(array).transpose()
    print('bezier.Curve')
    # help(bezier)
    curve = bezier.Curve(nodes, degree=degree)
    print('bezier:curve', curve)
    # ax = curve.plot(num_pts=256)

    # s_vals = np.linspace(curve.start, curve.end, 20)
    # points_eval = curve.evaluate_multi(s_vals)
    # x = points_eval.transpose()[0]
    # y = points_eval.transpose()[1]

    # plt.plot(x, y, 'o')
    # plt.plot(nodes.transpose()[0], nodes.transpose()[1], 'x')

    return curve


def getBFromInitialCurve(search_value, B, H):
    # Diese Funktion ermittelt den Ensprechenden B-Wert zu einem bestimmten H-Wert
    # in der Kurve gegeben durch B und H (Imports)
    index_right = np.where(search_value < H)[0][0]
    index_left = np.where(search_value > H)[0][-1]
    Bx = (search_value - H[index_left]) / (H[index_right] - H[index_left]) * (B[index_right] - B[index_left]) + B[
        index_left]
    return Bx


def funcBezier(B, H, lambda_B_knick, lambda_H_knick, B_knick, H_knick, degree, measured_initial_curve):
    print('funcBezier')
    curve = showBezierDegree(np.array([H[0], B[0]]), np.array([lambda_H_knick * H_knick, lambda_B_knick * B_knick]),
                             np.array([H[-1], B[-1]]), degree)

    curve_start = 0.0
    curve_end = 1.0
    points_eval = curve.evaluate_multi(np.linspace(curve_start, curve_end, 25))
    x = points_eval[0]
    y = points_eval[1]
    check_B = []
    check_H = []
    error = 0.0

    if (np.max(y[:-2]) >= y[-1]):
        return np.Inf

    # Routine zur Ermittlung des minimalen Fehlers:
    for i in range(len(points_eval[0]) - 2):
        check_B.append(getBFromInitialCurve(x[i + 1], measured_initial_curve["B"], measured_initial_curve["H"]))
        check_H.append(x[i + 1])
        error += (float(check_B[-1]) - y[i + 1]) ** 2

    # Routine zur Ermittlung des minimalen Fehlers unter Betrachtung des Integrals.
    # Die Energien unterhalb der B(H)-Kurven sollen gleich sein:
    check_H.append(x[-1])
    check_B.append(y[-1])

    check_B_Ar = np.array([0.0] + check_B)
    check_H_Ar = np.array([0.0] + check_H)

    deltaH = check_H_Ar[1:-1] - check_H_Ar[0:-2]
    deltaB_bezier = y[1:-1] - y[0:-2]
    deltaB_check = check_B_Ar[1:-1] - check_B_Ar[0:-2]

    integral_for_error = np.sum(0.5 * deltaH * deltaB_check) + np.sum(deltaH * check_B_Ar[0:-2])
    integral_for_error_bezier = np.sum(0.5 * deltaH * deltaB_bezier) + np.sum(deltaH * y[0:-2])

    error_integral = np.abs(integral_for_error_bezier - integral_for_error)

    return (error * 100 + 0 * error_integral)


def polishedBezier(initial_curve, measured_initial_curve, show_plots=False, filename_figure=None):
    # Steigungen im letzten und im ersten Punkt:
    # Ermittlung des Schnittpunktes der Vektoren die sich aus der Verbindung  der Messpunkte aus dem ersten und dem
    # letzten Paar von zwei aufeinanderfolgenden Messungen ergeben.
    # Dieser Schnittpunkte bildet den mittleren Punkt für die Bezier-Algorithmus.
    m_B_last_2 = (initial_curve["B"][-1] - initial_curve["B"][-2]) / (initial_curve["H"][-1] - initial_curve["H"][-2])
    b_knickgerade_2 = initial_curve["B"][-1] - m_B_last_2 * initial_curve["H"][-1]
    m_B_1 = (initial_curve["B"][1] - initial_curve["B"][0]) / (initial_curve["H"][1] - initial_curve["H"][0])
    H_knick = b_knickgerade_2 / (m_B_1 - m_B_last_2)
    B_knick = m_B_1 * b_knickgerade_2 / (m_B_1 - m_B_last_2)

    fehler = np.Inf
    Mu_0 = (4.0 * np.pi * 1e-7)
    # Mithilfe folgender Koeffizienten kann der Ort des mittleren Bezier-Punktes variiert werden, um eine höhere
    # Genauigkeit der approximierten Kurve zu erhalten.
    lambda_H_values = np.linspace(0.8, 1.15, 50)  # 0.95, 1.15, 15 # (0.80, 1.15, 30)
    lambda_B_values = np.linspace(0.8, 1.15, 50)  # 0.80, 1.02, 15 # (0.80, 1.15, 30)

    # Der Grad der Bezier-Annäherung wird hier variiert. Minimum ist "Anzahl der Punkte -1"
    grad_bezierFunc = np.min([5, len(initial_curve["B"]) - 1])

    # Erstellen der Threads:
    degrees = np.linspace(1, grad_bezierFunc, grad_bezierFunc)
    threads = []
    for degree in degrees:
        print('calc bezier fnc')
        thread = CalcBezierFunc(initial_curve, measured_initial_curve, lambda_B_values, lambda_H_values, degree,
                                H_knick, B_knick)
        threads += [thread]
        # print("Start Thread " + str(len(threads)))
        thread.start()

    for x in threads:
        x.join()
        print('x.err: ', x.err)
        print('fehler: ', fehler)
        if fehler > x.err:
            res_lambda_H = x.res_lambda_H
            res_lambda_B = x.res_lambda_B
            res_degree = int(x.degree)
            fehler = x.err

    fontP = FontProperties()
    fontP.set_size('small')

    print("  ... Threads beendet!")

    curve = showBezierDegree(np.array([initial_curve["H"][0], initial_curve["B"][0]]),
                             np.array([res_lambda_H * H_knick, res_lambda_B * B_knick]),
                             np.array([initial_curve["H"][-1], initial_curve["B"][-1]]), res_degree)  # res_degree+1

    curve_start = 0.0
    curve_end = 1.0
    points_eval = curve.evaluate_multi(np.linspace(curve_start, curve_end, 50))
    h = points_eval[0]
    b = points_eval[1]
    if h[0] == 0.0:
        mu = b[1:] / h[1:] / Mu_0
        mu = np.append(mu[0], mu)
    else:
        mu = b / h / Mu_0

    # mu_dirr = (b[1:] - b[:-1]) / (h[1:] - h[:-1]) / Mu_0
    # mu_dirr = np.append(mu_dirr[0], mu_dirr)

    if show_plots == True:
        fig = plt.figure()
        ax2 = fig.add_subplot(211)
        ax2.grid(True)
        ax2.set_title("Initial Curve for FE simulation vs. measurement")
        ax2.set_xlabel(r"$H [A/m]$")
        ax2.set_ylabel(r"$B [T]$")
        ax2.set_xlim(0, np.max([max(measured_initial_curve["H"]), np.max(h)]) * 1.1)
        ax2.set_ylim(0, np.max([max(measured_initial_curve["B"]), np.max(b)]) * 1.1)
        plt.plot(measured_initial_curve["H"][1:], measured_initial_curve["B"][1:], 'b')
        plt.plot(measured_initial_curve["H"][1:], measured_initial_curve["B"][1:], 'go')
        plt.plot(h, b, 'r')
        plt.plot(h, b, 'x')
        ax2.legend(["measured curve", "measured points", "bezier curve"], prop=fontP, loc='lower right')
        box = ax2.get_position()
        ax2.set_position([box.x0, box.y0, box.width, 0.85 * box.height])

        ax = fig.add_subplot(212)
        ax.grid(True)
        ax.set_xlabel(r"$B [T]$")
        ax.set_ylabel(r"$\mu_{r} [1]$")
        ax.set_xlim(0, np.max([max(measured_initial_curve["B"]), np.max(b)]) * 1.1)
        ax.set_ylim(0, np.max([max(measured_initial_curve["mu_r"]), np.max(mu)]) * 1.1)
        plt.plot(measured_initial_curve["B"][1:], measured_initial_curve["mu_r"][1:], 'b')
        plt.plot(measured_initial_curve["B"][1:], measured_initial_curve["mu_r"][1:], 'go')
        plt.plot(b, mu, 'r')
        ax.legend(["measured curve", "measured points", "bezier curve"], prop=fontP, loc='lower left')
        box = ax.get_position()
        ax.set_position([box.x0, box.y0, box.width, 0.85 * box.height])

        try:
            my_path = os.path.abspath(
                __file__)  # Figures out the absolute path for you in case your working directory moves around.
            if not filename_figure == None:
                my_file = filename_figure + "_SimCurve_" + str(int(measured_initial_curve["f"])) + "Hz.png"
            else:
                my_file = "\\bezier_curves_" + str(int(measured_initial_curve["f"])) + "Hz.png"
            fig.savefig(my_file)
            print("Create figure for " + str(measured_initial_curve["f"]) + " Hz: " + my_file)
        except:
            print("Error. No figure created!")
        fig.show()

    return {"B": b, "mu_r": mu, "H": h, "bezier_values": [res_degree, res_lambda_H * H_knick, res_lambda_B * B_knick]}


def func(x, c_a, c_b, n, mu_i):
    # mu_r = 0        # Relative magnetic permeability of ferromagnetic flux tube element
    # mu_i = 230        # Initial relative permeability at B=0
    # B = x           # Flux density in ferromagnetic flux tube element
    B_myMax = 0.446  # Flux density at maximum relative permeability
    # N = n           # Exponent of approximation function
    # B_N = np.abs(B / B_myMax)
    mu_r = 1 + (mu_i - 1 + c_a * np.abs(x / B_myMax)) / (1 + c_b * np.abs(x / B_myMax) * np.abs(x / B_myMax) ** n)
    return mu_r


def func2(x, a, b, c, d, n, p):
    mu_r = p * (0.5 - ((1 - n) * np.arctan(a * x - b) - n * np.arctan(c * x - d)) / np.pi)
    return mu_r


def func3(x, a, b, c, d, e):
    mu_r = a - (b * np.arctan(c * (x - d)) / np.pi) + e * x
    return mu_r


def func4(x, a, b, c, d):
    mu_r = a / (np.exp(b * (x - c)) + 1) + d
    return mu_r


def func5(x, a, b, c, d, g):
    gg = int(g)
    print(gg)
    mu_r = a / (np.exp(b * ((x - c) ** gg * 0.4) + 1)) + d
    return mu_r


def writeMaterialsFile(name, frequency, all_initial_curves, khfe, kefe, rho, path, k=1.0):
    # Write Material XML-File:
    material_database = etree.Element("MaterialDatabase")
    material_class = etree.SubElement(material_database, "MaterialClass", Name="Ferromagnetic material", ID="1")
    material = etree.SubElement(material_class, "Material", Name=name, ID="359")

    for freq in frequency:

        dataset = etree.SubElement(material, "Dataset", Name="Dataset_" + str(freq) + "_Hz", ID=str(int(freq)))
        properties = etree.SubElement(dataset, "Properties")

        diagrams = etree.SubElement(dataset, "Diagrams")
        # if len(all_initial_curves) > 1:
        initial_curve = all_initial_curves.pop(0)
        b_values = str(initial_curve["B"][0] * k)
        h_values = str(initial_curve["H"][0])

        for i in range(len(initial_curve["B"]) - 1):
            b_values += (";" + str(initial_curve["B"][i + 1] * k))
            h_values += (";" + str(initial_curve["H"][i + 1]))

        # Materialdaten:

        # Color Red
        item = etree.SubElement(properties, "item")
        etree.SubElement(item, "Name").text = "colorR"
        etree.SubElement(item, "Description").text = "ColorR"
        etree.SubElement(item, "Group").text = "Other"
        etree.SubElement(item, "Value").text = "211.0"
        etree.SubElement(item, "Unit").text = ""
        etree.SubElement(item, "Tex").text = "RGB red"
        etree.SubElement(item, "Locked").text = "false"
        etree.SubElement(item, "Fixed").text = "false"

        # Color Blue
        item = etree.SubElement(properties, "item")
        etree.SubElement(item, "Name").text = "colorB"
        etree.SubElement(item, "Description").text = "ColorB"
        etree.SubElement(item, "Group").text = "Other"
        etree.SubElement(item, "Value").text = "211.0"
        etree.SubElement(item, "Unit").text = ""
        etree.SubElement(item, "Tex").text = "RGB blue"
        etree.SubElement(item, "Locked").text = "false"
        etree.SubElement(item, "Fixed").text = "false"

        # Color Green
        item = etree.SubElement(properties, "item")
        etree.SubElement(item, "Name").text = "colorG"
        etree.SubElement(item, "Description").text = "ColorG"
        etree.SubElement(item, "Group").text = "Other"
        etree.SubElement(item, "Value").text = "211.0"
        etree.SubElement(item, "Unit").text = ""
        etree.SubElement(item, "Tex").text = "RGB green"
        etree.SubElement(item, "Locked").text = "false"
        etree.SubElement(item, "Fixed").text = "false"

        # kefe
        item = etree.SubElement(properties, "item")
        etree.SubElement(item, "Name").text = "kefe"
        etree.SubElement(item, "Description").text = "Eddy current loss factor"
        etree.SubElement(item, "Group").text = "Magnetic"
        etree.SubElement(item, "Value").text = str(kefe)
        etree.SubElement(item, "Unit").text = "W.kg-1"
        etree.SubElement(item, "Tex").text = r"k_{e,fe}"
        etree.SubElement(item, "Locked").text = "true"
        etree.SubElement(item, "Fixed").text = "false"

        # khfe
        item = etree.SubElement(properties, "item")
        etree.SubElement(item, "Name").text = "khfe"
        etree.SubElement(item, "Description").text = "Hysteresis loss factor"
        etree.SubElement(item, "Group").text = "Magnetic"
        etree.SubElement(item, "Value").text = str(khfe)
        etree.SubElement(item, "Unit").text = "W.kg-1"
        etree.SubElement(item, "Tex").text = r"k_{h,fe}"
        etree.SubElement(item, "Locked").text = "true"
        etree.SubElement(item, "Fixed").text = "false"

        # rho
        item = etree.SubElement(properties, "item")
        etree.SubElement(item, "Name").text = "rho"
        etree.SubElement(item, "Description").text = "Density"
        etree.SubElement(item, "Group").text = "Mechanic"
        etree.SubElement(item, "Value").text = str(rho)
        etree.SubElement(item, "Unit").text = "kg.dm-3"
        etree.SubElement(item, "Tex").text = r"\rho"
        etree.SubElement(item, "Locked").text = "true"
        etree.SubElement(item, "Fixed").text = "false"

        # k
        item = etree.SubElement(properties, "item")
        etree.SubElement(item, "Name").text = "k"
        etree.SubElement(item, "Description").text = "Lamination fill factor"
        etree.SubElement(item, "Group").text = "Mechanic"
        etree.SubElement(item, "Value").text = str(k)
        etree.SubElement(item, "Unit").text = "1"
        etree.SubElement(item, "Tex").text = "k"
        etree.SubElement(item, "Locked").text = "true"
        etree.SubElement(item, "Fixed").text = "false"

        # B(H)-Kennlinie:

        item = etree.SubElement(diagrams, "item")
        etree.SubElement(item, "Name").text = "HvsB"
        etree.SubElement(item, "Description").text = "BH-curve"

        axis = etree.SubElement(item, "Axis")

        item = etree.SubElement(axis, "item")
        etree.SubElement(item, "Name").text = "H"
        etree.SubElement(item, "Description").text = "Coercive field strength"
        etree.SubElement(item, "Group").text = "diagram"
        etree.SubElement(item, "Values").text = h_values  # "0.0;124.1100006;171.5700073;120000.0"
        etree.SubElement(item, "Unit").text = "A.m-1"
        etree.SubElement(item, "Tex").text = "H"
        etree.SubElement(item, "Locked").text = "true"
        etree.SubElement(item, "Fixed").text = "false"

        item = etree.SubElement(axis, "item")
        etree.SubElement(item, "Name").text = "B"
        etree.SubElement(item, "Description").text = "Magnetic flux density"
        etree.SubElement(item, "Group").text = "diagram"
        etree.SubElement(item, "Values").text = b_values  # "0.0;0.095488998;2.046399238;2.0709921;2.098932998"
        etree.SubElement(item, "Unit").text = "T"
        etree.SubElement(item, "Tex").text = "B"
        etree.SubElement(item, "Locked").text = "true"
        etree.SubElement(item, "Fixed").text = "false"

    tree = etree.ElementTree(material_database)
    tree.write(path)  # "singleMaterial.mat"

    return 1


class start_bh:

    @staticmethod
    def process_bhdata(material_dictionary, show_plots=True):
        name = material_dictionary['name']
        kh = material_dictionary['kh']
        ke = material_dictionary['ke']
        c = material_dictionary['stackfactor']  # Stapelfaktor
        density = material_dictionary['density']

        meas = sio.loadmat(r"D:\01_Projekte\04_Material\03_Steel\01_BH-Kennlinien\02_Gereon\NO30-15\NO30-15.mat")
        H = np.array(meas["values"][0][0]["H"][0])
        initial_curveMod = []
        measurements = []
        for i in range(len(meas["values"][0])):
            try:
                mu_r = np.array(meas["values"][0][i]["B"][0][1:]) / np.array(meas["values"][0][i]["H"][0][1:]) / (
                            4.0 * np.pi * 1e-7)
                mu_r = np.append(mu_r[0], mu_r)
                measurements.append({"f": meas["values"][0][i]["f"][0][0], "H": meas["values"][0][i]["H"][0],
                                     "B": meas["values"][0][i]["B"][0], "mu_r": mu_r})
                # plt.plot(measurements[-1]["H"], measurements[-1]["B"])
            except:
                pass
        frequencies = []
        max_mu_r = 0.0
        max_b = 0.0
        max_h = 0.0
        for index in range(len(measurements)):
            initial_curve_one = calcInitualBHcurveConstEnergy(measurements[index])
            initial_curve_two = copy.deepcopy(initial_curve_one)
            for _ in range(len(initial_curve_one["B"])):
                # Schleife, die solange läuft, bis die ganze Kurve nur noch streng monoton fallend ist
                initial_curve_two_res = checkStrictlyDecreasing(
                    copy.deepcopy(initial_curve_two))
                if initial_curve_two == initial_curve_two_res:
                    break
                else:
                    initial_curve_two = initial_curve_two_res
            # initial_curveMod.append(interfaceMagOptMaterial.polished(interfaceMagOptMaterial.polishedByCircle(copy.deepcopy(initial_curve_two))))
            print("Index " + str(index + 1))
            initial_curveMod.append(
                polishedBezier(copy.deepcopy(initial_curve_two), copy.deepcopy(measurements[index]), show_plots,
                               '\Mat_' + name))
            frequencies.append(str(measurements[index]["f"]) + " Hz")
            max_mu_r = np.max([max_mu_r, np.max(initial_curveMod[-1]["mu_r"])])
            max_b = np.max([max_b, np.max(initial_curveMod[-1]["B"])])
            max_h = np.max([max_h, np.max(initial_curveMod[-1]["H"])])

        fontP = FontProperties()
        fontP.set_size('small')
        fig2 = plt.figure()

        ax1 = fig2.add_subplot(311)
        ax1.grid(True)
        ax1.set_title(
            r'$Initial \ Curve \ for \ FE \ simulation \ vs. \ measurement$')  # #"Initial Curve for FE simulation")
        ax1.set_xlabel(r"$H [A/m]$")
        ax1.set_ylabel(r"$B [T]$")
        ax1.set_xlim(0, max_h * 1.1)
        ax1.set_ylim(0, max_b * 1.1)
        for i in range(len(initial_curveMod)):
            plt.plot(initial_curveMod[i]["H"], initial_curveMod[i]["B"])
        box = ax1.get_position()
        ax1.set_position([box.x0, box.y0, box.width * 0.8, box.height])
        ax1.legend(frequencies, prop=fontP, loc='center left', bbox_to_anchor=(1, 0.5))

        ax3 = fig2.add_subplot(312)
        ax3.grid(True)
        ax3.set_xlabel(r"$B [T]$")
        ax3.set_ylabel(r"$\mu_{r} [1]$")
        ax3.set_xlim(0, max_b * 1.1)
        ax3.set_ylim(0, max_mu_r * 1.1)
        for i in range(len(initial_curveMod)):
            plt.plot(initial_curveMod[i]["B"], initial_curveMod[i]["mu_r"])
        box = ax3.get_position()
        ax3.set_position([box.x0, box.y0, box.width * 0.8, box.height])
        ax3.legend(frequencies, prop=fontP, loc='center left', bbox_to_anchor=(1, 0.5))

        ax4 = fig2.add_subplot(313)
        ax4.grid(True)
        ax4.set_xlabel(r"$B [T]$")
        ax4.set_ylabel(r"$\mu_{r,diff} [1]$")
        ax4.set_xlim(0, max_b * 1.1)
        ax4.set_ylim(0, max_mu_r * 1.1)

        for i in range(len(initial_curveMod)):
            mu_dirr = (initial_curveMod[i]["B"][1:] - initial_curveMod[i]["B"][:-1]) / (
                        initial_curveMod[i]["H"][1:] - initial_curveMod[i]["H"][:-1]) / (4.0 * np.pi * 1e-7)
            mu_dirr = np.append(mu_dirr[0], mu_dirr)
            plt.plot(initial_curveMod[i]["B"], mu_dirr)
        box = ax4.get_position()
        ax4.set_position([box.x0, box.y0, box.width * 0.8, box.height])
        ax4.legend(frequencies, prop=fontP, loc='center left', bbox_to_anchor=(1, 0.5))

        sio.savemat('\Mat_' + name + '_SimCurves.mat', mdict={'vals': initial_curveMod})

        fig2.show()
        fig2.savefig("\Mat_" + name + "_SimCurves.pdf")

        fname = '\Mat_' + name + '_MagOpt.mat'

        writeMaterialsFile(name, [50], initial_curveMod, kh, ke, density, fname, c)


class BHCurve_correction:
    @staticmethod
    def correct_bhcurve(material_dictionary, H, B, show_plots=True):
        name = material_dictionary['name']

        mu_0 = 4 * np.pi * 1e-7
        # H = material_dictionary['Magnetic field strength']['value'][0]
        # B = [a + b for a, b in zip([j / 1000.0 for j in material_dictionary['Magnetic polarization']['value'][0]],
        #                            [h * mu_0 for h in material_dictionary['Magnetic field strength']['value'][0]])]
        # H = material_dictionary['H_extrapolated'][0]
        # B = material_dictionary['B_extrapolated'][0]

        initial_curveMod = []
        measurements = []

        try:
            mu_r = np.array(B[1:]) / np.array(H[1:]) / mu_0
            mu_r = np.append(mu_r[0], mu_r)
            measurements.append({"f": material_dictionary['Frequency']['value'][0], "H": H, "B": B, "mu_r": mu_r})
            # plt.plot(measurements[-1]["H"], measurements[-1]["B"])
        except:
            pass

        frequencies = []
        max_mu_r = 0.0
        max_b = 0.0
        max_h = 0.0
        for index in range(len(measurements)):
            initial_curve_one = calcInitualBHcurveConstEnergy(measurements[index])
            initial_curve_two = copy.deepcopy(initial_curve_one)
            for _ in range(len(initial_curve_one["B"])):
                # for-loop adjusting the curve, such that it is strictly monotonously decreasing
                initial_curve_two_res = checkStrictlyDecreasing(copy.deepcopy(initial_curve_two))

                if initial_curve_two == initial_curve_two_res:
                    break
                else:
                    initial_curve_two = initial_curve_two_res

            # initial_curveMod.append(interfaceMagOptMaterial.polished(interfaceMagOptMaterial.polishedByCircle(copy.deepcopy(initial_curve_two))))
            print("Index " + str(index + 1))
            initial_curveMod.append(polishedBezier(copy.deepcopy(initial_curve_two), copy.deepcopy(measurements[index]), show_plots, '\Mat_' + name))
            frequencies.append(str(measurements[index]["f"]) + " Hz")
            max_mu_r = np.max([max_mu_r, np.max(initial_curveMod[-1]["mu_r"])])
            max_b = np.max([max_b, np.max(initial_curveMod[-1]["B"])])
            max_h = np.max([max_h, np.max(initial_curveMod[-1]["H"])])

        if show_plots:
            fontP = FontProperties()
            fontP.set_size('small')
            fig2 = plt.figure()

            ax1 = fig2.add_subplot(311)
            ax1.grid(True)
            ax1.set_title( r'$Initial \ Curve \ for \ FE \ simulation \ vs. \ measurement$')  #"Initial Curve for FE simulation")
            ax1.set_xlabel(r"$H [A/m]$")
            ax1.set_ylabel(r"$B [T]$")
            ax1.set_xlim(0, max_h * 1.1)
            ax1.set_ylim(0, max_b * 1.1)
            for i in range(len(initial_curveMod)):
                plt.plot(initial_curveMod[i]["H"], initial_curveMod[i]["B"])
            box = ax1.get_position()
            ax1.set_position([box.x0, box.y0, box.width * 0.8, box.height])
            ax1.legend(frequencies, prop=fontP, loc='center left', bbox_to_anchor=(1, 0.5))

            ax3 = fig2.add_subplot(312)
            ax3.grid(True)
            ax3.set_xlabel(r"$B [T]$")
            ax3.set_ylabel(r"$\mu_{r} [1]$")
            ax3.set_xlim(0, max_b * 1.1)
            ax3.set_ylim(0, max_mu_r * 1.1)
            for i in range(len(initial_curveMod)):
                plt.plot(initial_curveMod[i]["B"], initial_curveMod[i]["mu_r"])
            box = ax3.get_position()
            ax3.set_position([box.x0, box.y0, box.width * 0.8, box.height])
            ax3.legend(frequencies, prop=fontP, loc='center left', bbox_to_anchor=(1, 0.5))

            ax4 = fig2.add_subplot(313)
            ax4.grid(True)
            ax4.set_xlabel(r"$B [T]$")
            ax4.set_ylabel(r"$\mu_{r,diff} [1]$")
            ax4.set_xlim(0, max_b * 1.1)
            ax4.set_ylim(0, max_mu_r * 1.1)

            for i in range(len(initial_curveMod)):
                mu_dirr = (initial_curveMod[i]["B"][1:] - initial_curveMod[i]["B"][:-1]) / (
                            initial_curveMod[i]["H"][1:] - initial_curveMod[i]["H"][:-1]) / (4.0 * np.pi * 1e-7)
                mu_dirr = np.append(mu_dirr[0], mu_dirr)
                plt.plot(initial_curveMod[i]["B"], mu_dirr)
            box = ax4.get_position()
            ax4.set_position([box.x0, box.y0, box.width * 0.8, box.height])
            ax4.legend(frequencies, prop=fontP, loc='center left', bbox_to_anchor=(1, 0.5))

            sio.savemat('\Mat_' + name + '_SimCurves.mat', mdict={'vals': initial_curveMod})

            fig2.show()
            fig2.savefig("\Mat_" + name + "_SimCurves.pdf")

        fname = '\Mat_' + name + '_MagOpt.mat'

        kh = material_dictionary['khfe']
        ke = material_dictionary['kefe']

        H_corrected = initial_curveMod[-1]["H"]
        B_corrected = initial_curveMod[-1]["B"]

        writeMaterialsFile(name, [50], initial_curveMod, kh, ke, 7.60, fname)

        print('fertig')

        return H_corrected, B_corrected

class plot_curves:
    @staticmethod
    def save_figures(material_dictionary):
        max_h = material_dictionary['H_extrapolated'][0][-1]
        max_b = material_dictionary['B_extrapolated'][0][-1]

        max_ho = material_dictionary['H_original'][0][-1]
        max_bo = material_dictionary['B_original'][0][-1]

        fig = plt.figure()
        ax_bh = fig.add_subplot(211)
        ax_bh.grid(True)
        ax_bh.set_title("Initial Curve for FE simulation vs. measurement")
        ax_bh.set_xlabel(r"$H [A/m]$")
        ax_bh.set_ylabel(r"$B [T]$")
        ax_bh.set_xlim(0, max_h * 1.1)
        ax_bh.set_ylim(0, max_b * 1.1)
        plt.plot(material_dictionary['H_original'][0], material_dictionary['B_original'][0], "ro")
        plt.plot(material_dictionary['H_extrapolated'][0], material_dictionary['B_extrapolated'][0], "b-")
        plt.plot(material_dictionary['H_corrected'][0], material_dictionary['B_corrected'][0], "g-")
        ax_bh.legend(["measured points", "extrapolated curve", "corrected curve"], prop=fontP, loc='lower right')
        box = ax_bh.get_position()
        ax_bh.set_position([box.x0, box.y0, box.width, 0.85 * box.height])

        fig_bh = plt.figure()
        ax_bh = fig_bh.add_subplot(212)
        ax_bh.grid(True)
        ax_bh.set_title("Zoom BH-Curve")
        ax_bh.set_xlabel(r"$H [A/m]$")
        ax_bh.set_ylabel(r"$B [T]$")
        ax_bh.set_xlim(0, max_ho * 1.1)
        ax_bh.set_ylim(0, max_bo * 1.1)
        plt.plot(material_dictionary['H_original'][0], material_dictionary['B_original'][0], "ro")
        plt.plot(material_dictionary['H_extrapolated'][0], material_dictionary['B_extrapolated'][0], "b-")
        plt.plot(material_dictionary['H_corrected'][0], material_dictionary['B_corrected'][0], "g-")
        ax_bh.legend(["measured points", "extrapolated curve", "corrected curve"], prop=fontP, loc='lower right')
        box = ax_bh.get_position()
        ax_bh.set_position([box.x0, box.y0, box.width, 0.85 * box.height])

        try:
            my_path = os.path.abspath(
                __file__)  # Figures out the absolute path for you in case your working directory moves around.
            if not filename_figure == None:
                my_file = filename_figure + "_BHCurve_" + str(int(measured_initial_curve["f"])) + "Hz.png"
            else:
                my_file = "\\bhcurves_" + str(int(measured_initial_curve["f"])) + "Hz.png"
                fig_bh.savefig(my_file)
                print("Create figure for " + str(measured_initial_curve["f"]) + " Hz: " + my_file)
        except:
            print("Error. No figure created!")

        # fig_mu = plt.figure()
        # ax_mu = fig_mu.add_subplot(211)
        # ax_mu.grid(True)
        # ax_mu.set_xlabel(r"$B [T]$")
        # ax_mu.set_ylabel(r"$\mu_{r} [1]$")
        # ax_mu.set_xlim(0, np.max([max(measured_initial_curve["B"]), np.max(b)]) * 1.1)
        # ax_mu.set_ylim(0, np.max([max(measured_initial_curve["mu_r"]), np.max(mu)]) * 1.1)
        # plt.plot(measured_initial_curve["B"][1:], measured_initial_curve["mu_r"][1:], 'b')
        # plt.plot(measured_initial_curve["B"][1:], measured_initial_curve["mu_r"][1:], 'go')
        # plt.plot(b, mu, 'r')
        # ax_mu.legend(["measured curve", "measured points", "bezier curve"], prop=fontP, loc='lower left')
        # box = ax_mu.get_position()
        # ax_mu.set_position([box.x0, box.y0, box.width, 0.85 * box.height])

        # ax = fig.add_subplot(212)
        # ax.grid(True)
        # ax.set_xlabel(r"$B [T]$")
        # ax.set_ylabel(r"$\mu_{r} [1]$")
        # ax.set_xlim(0, np.max([max(measured_initial_curve["B"]), np.max(b)]) * 1.1)
        # ax.set_ylim(0, np.max([max(measured_initial_curve["mu_r"]), np.max(mu)]) * 1.1)
        # plt.plot(measured_initial_curve["B"][1:], measured_initial_curve["mu_r"][1:], 'b')
        # plt.plot(measured_initial_curve["B"][1:], measured_initial_curve["mu_r"][1:], 'go')
        # plt.plot(b, mu, 'r')
        # ax.legend(["measured curve", "measured points", "bezier curve"], prop=fontP, loc='lower left')
        # box = ax.get_position()
        # ax.set_position([box.x0, box.y0, box.width, 0.85 * box.height])
        #
        # try:
        #     my_path = os.path.abspath(
        #         __file__)  # Figures out the absolute path for you in case your working directory moves around.
        #     if not filename_figure == None:
        #         my_file = filename_figure + "_SimCurve_" + str(int(measured_initial_curve["f"])) + "Hz.png"
        #     else:
        #         my_file = "\\bezier_curves_" + str(int(measured_initial_curve["f"])) + "Hz.png"
        #     fig.savefig(my_file)
        #     print("Create figure for " + str(measured_initial_curve["f"]) + " Hz: " + my_file)
        # except:
        #     print("Error. No figure created!")

if __name__ == "__main__":

    name = "NO30-15"
    kh = 2.6291
    ke = 0.2680
    c = 0.96  # Stapelfaktor

    meas = sio.loadmat(r"NO30-15\NO30-15.mat")
    H = np.array(meas["values"][0][0]["H"][0])
    initial_curveMod = []
    measurements = []
    for i in range(len(meas["values"][0])):
        try:
            mu_r = np.array(meas["values"][0][i]["B"][0][1:]) / np.array(meas["values"][0][i]["H"][0][1:]) / (
                        4.0 * np.pi * 1e-7)
            mu_r = np.append(mu_r[0], mu_r)
            print("f", meas["values"][0][i]["f"][0][0])
            print("H", meas["values"][0][i]["H"][0])
            print("B", meas["values"][0][i]["B"][0])
            print("mu_r", mu_r)
            measurements.append({"f": meas["values"][0][i]["f"][0][0], "H": meas["values"][0][i]["H"][0],
                                 "B": meas["values"][0][i]["B"][0], "mu_r": mu_r})
            # plt.plot(measurements[-1]["H"], measurements[-1]["B"])
        except:
            pass
    frequencies = []
    max_mu_r = 0.0
    max_b = 0.0
    max_h = 0.0
    for index in range(len(measurements)):
        initial_curve_one = calcInitualBHcurveConstEnergy(measurements[index])
        initial_curve_two = copy.deepcopy(initial_curve_one)
        for _ in range(len(initial_curve_one["B"])):
            # Schleife, die solange läuft, bis die ganze Kurve nur noch streng monoton fallend ist
            initial_curve_two_res = checkStrictlyDecreasing(copy.deepcopy(initial_curve_two))

            if initial_curve_two == initial_curve_two_res:
                break
            else:
                initial_curve_two = initial_curve_two_res
        # initial_curveMod.append(interfaceMagOptMaterial.polished(interfaceMagOptMaterial.polishedByCircle(copy.deepcopy(initial_curve_two))))
        print("Index " + str(index + 1))
        initial_curveMod.append(
            polishedBezier(copy.deepcopy(initial_curve_two), copy.deepcopy(measurements[index]), True, '\Mat_' + name))
        frequencies.append(str(measurements[index]["f"]) + " Hz")
        max_mu_r = np.max([max_mu_r, np.max(initial_curveMod[-1]["mu_r"])])
        max_b = np.max([max_b, np.max(initial_curveMod[-1]["B"])])
        max_h = np.max([max_h, np.max(initial_curveMod[-1]["H"])])

    fontP = FontProperties()
    fontP.set_size('small')
    fig2 = plt.figure()

    ax1 = fig2.add_subplot(311)
    ax1.grid(True)
    ax1.set_title(
        r'$Initial \ Curve \ for \ FE \ simulation \ vs. \ measurement$')  # #"Initial Curve for FE simulation")
    ax1.set_xlabel(r"$H [A/m]$")
    ax1.set_ylabel(r"$B [T]$")
    ax1.set_xlim(0, max_h * 1.1)
    ax1.set_ylim(0, max_b * 1.1)
    for i in range(len(initial_curveMod)):
        plt.plot(initial_curveMod[i]["H"], initial_curveMod[i]["B"])
    box = ax1.get_position()
    ax1.set_position([box.x0, box.y0, box.width * 0.8, box.height])
    ax1.legend(frequencies, prop=fontP, loc='center left', bbox_to_anchor=(1, 0.5))

    ax3 = fig2.add_subplot(312)
    ax3.grid(True)
    ax3.set_xlabel(r"$B [T]$")
    ax3.set_ylabel(r"$\mu_{r} [1]$")
    ax3.set_xlim(0, max_b * 1.1)
    ax3.set_ylim(0, max_mu_r * 1.1)
    for i in range(len(initial_curveMod)):
        plt.plot(initial_curveMod[i]["B"], initial_curveMod[i]["mu_r"])
    box = ax3.get_position()
    ax3.set_position([box.x0, box.y0, box.width * 0.8, box.height])
    ax3.legend(frequencies, prop=fontP, loc='center left', bbox_to_anchor=(1, 0.5))

    ax4 = fig2.add_subplot(313)
    ax4.grid(True)
    ax4.set_xlabel(r"$B [T]$")
    ax4.set_ylabel(r"$\mu_{r,diff} [1]$")
    ax4.set_xlim(0, max_b * 1.1)
    ax4.set_ylim(0, max_mu_r * 1.1)

    ###############
    def bhcurve(h, alpha, beta):
        b = h / (alpha + beta * h)
        return b


    popt0, pcov0 = curve_fit(bhcurve, initial_curveMod[i]["H"][-30:-20], initial_curveMod[i]["B"][-30:-20], p0=[2, 2])
    print('popt0', popt0)
    popt10, pcov10 = curve_fit(bhcurve, initial_curveMod[i]["H"][-10:-1], initial_curveMod[i]["B"][-10:-1], p0=[2, 2])
    print('popt10', popt10)
    popt20, pcov20 = curve_fit(bhcurve, initial_curveMod[i]["H"][-20:-1], initial_curveMod[i]["B"][-20:-1], p0=[2, 2])
    print('popt20', popt20)
    popt30, pcov30 = curve_fit(bhcurve, initial_curveMod[i]["H"][-30:-1], initial_curveMod[i]["B"][-30:-1], p0=[2, 2])
    print('popt30', popt30)
    popt40, pcov40 = curve_fit(bhcurve, initial_curveMod[i]["H"][-40:-1], initial_curveMod[i]["B"][-40:-1], p0=[2, 2])
    print('popt40', popt40)
    popt50, pcov50 = curve_fit(bhcurve, initial_curveMod[i]["H"][-50:-1], initial_curveMod[i]["B"][-50:-1], p0=[2, 2])
    print('popt50', popt50)
    popt60, pcov60 = curve_fit(bhcurve, initial_curveMod[i]["H"][-60:-1], initial_curveMod[i]["B"][-60:-1], p0=[2, 2])
    print('popt60', popt60)
    popt70, pcov70 = curve_fit(bhcurve, initial_curveMod[i]["H"][-70:-1], initial_curveMod[i]["B"][-70:-1], p0=[2, 2])
    print('popt70', popt70)
    popt80, pcov80 = curve_fit(bhcurve, initial_curveMod[i]["H"][-80:-1], initial_curveMod[i]["B"][-80:-1], p0=[2, 2])
    print('popt80', popt80)

    plt.figure(figsize=(6, 4))
    plt.scatter(initial_curveMod[i]["H"], initial_curveMod[i]["B"], label='Data')
    plt.scatter(initial_curveMod[i]["H"][-20:-1], initial_curveMod[i]["B"][-20:-1], c='r', label='Data set')
    plt.plot(initial_curveMod[i]["H"], bhcurve(initial_curveMod[i]["H"], popt0[0], popt0[1]), 'k', label='Fitted function 0')
    plt.plot(initial_curveMod[i]["H"], bhcurve(initial_curveMod[i]["H"], popt10[0], popt10[1]), 'r', label='Fitted function 10')
    plt.plot(initial_curveMod[i]["H"], bhcurve(initial_curveMod[i]["H"], popt20[0], popt20[1]), 'g', label='Fitted function 20')
    plt.plot(initial_curveMod[i]["H"], bhcurve(initial_curveMod[i]["H"], popt30[0], popt30[1]), 'b', label='Fitted function 30')
    plt.plot(initial_curveMod[i]["H"], bhcurve(initial_curveMod[i]["H"], popt40[0], popt40[1]), 'c', label='Fitted function 40')
    plt.plot(initial_curveMod[i]["H"], bhcurve(initial_curveMod[i]["H"], popt50[0], popt50[1]), 'k', label='Fitted function 50')
    plt.plot(initial_curveMod[i]["H"], bhcurve(initial_curveMod[i]["H"], popt60[0], popt60[1]), 'y', label='Fitted function 60')
    plt.plot(initial_curveMod[i]["H"], bhcurve(initial_curveMod[i]["H"], popt70[0], popt70[1]), 'm', label='Fitted function 70')
    plt.plot(initial_curveMod[i]["H"], bhcurve(initial_curveMod[i]["H"], popt80[0], popt80[1]), 'k', label='Fitted function 80')

    plt.legend(loc='best')

    plt.show()
    ###############

    for i in range(len(initial_curveMod)):
        mu_dirr = (initial_curveMod[i]["B"][1:] - initial_curveMod[i]["B"][:-1]) / (
                    initial_curveMod[i]["H"][1:] - initial_curveMod[i]["H"][:-1]) / (4.0 * np.pi * 1e-7)
        mu_dirr = np.append(mu_dirr[0], mu_dirr)
        plt.plot(initial_curveMod[i]["B"], mu_dirr)
    box = ax4.get_position()
    ax4.set_position([box.x0, box.y0, box.width * 0.8, box.height])
    ax4.legend(frequencies, prop=fontP, loc='center left', bbox_to_anchor=(1, 0.5))

    sio.savemat('\Mat_' + name + '_SimCurves.mat', mdict={'vals': initial_curveMod})

    fig2.show()
    fig2.savefig("\Mat_" + name + "_SimCurves.pdf")

    fname = '\Mat_' + name + '_MagOpt.mat'

    writeMaterialsFile(name, [50], initial_curveMod, kh, ke, 7.60, fname, c)

    print('fertig')