#!/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 5794 $
$LastChangedDate:: 2022-04-21 18:17:41 +0200#$

This file is part of SyMSpace.

Copyright (c) 2018, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import glob
import os
import optparse
import datetime
from os.path import join as pathjoin
from symspace.osutils.pathutils import Paths
from symspace.osutils.pathutils import CMDParser
from symspace.osutils.multiprocessing import ExternalMultiprocessing
from componentspace.report.json import JSONReport
from componentspace.report.junit import JUnitReport
from componentspace.create_buildscript import ComponentBuilder
from componentspace.testing.scripts import build_external_component_test
from componentspace.testing.abstract_test_builder import AbstractTestBuilder


class ComponentTestBuilder(AbstractTestBuilder):
    mop_file_ending = '.mop'
    build_reports_dir = os.path.join(Paths.getCSTestReportsDir(), 'build_component_tests')
    run_reports_dir = os.path.join(Paths.getCSTestReportsDir(), 'run_component_tests')

    @classmethod
    def get_search_pattern(cls):
        search_pattern = '\\**\\test_*{}'.format(cls.mop_file_ending)
        return search_pattern

    # Todo remove start_path from __init__
    def __init__(self, start_path: str, full_build: bool, timeout: float = 300):
        """
        Class collects and builds SyMProject tests for components from scripts

        Parameters
        ----------
        start_path : str
            Path to the directory where build starts.
            Components in all subdirectories will be built.
        full_build : bool
            If True all components are built
        timeout : float
            Timeout (in seconds) for single component test build script
        """
        ''' unittest files'''
        build_results_xml = pathjoin(start_path, 'build_component_test_results-junit.xml')
        junit_build = JUnitReport('componentspace.create_tests.Build', build_results_xml, '')

        super().__init__(start_path, timeout, junit_build=junit_build)
        self.full_build = full_build
        self.build_freecad = True
        self.build_creo = True

        '''JSON file with last versions of last build'''
        self.build_info = pathjoin(start_path, 'build_info.json')

        if not os.path.isfile(self.build_info):
            builder = ComponentBuilder(start_path, False)

            build_info = builder.writeJSONReport()
            if not os.path.samefile(build_info, self.build_info):
                raise Exception("Files are not the same:\n\t{}\n\t{}".format(build_info, self.build_info))

        if not os.path.isfile(self.build_info):
            raise FileNotFoundError('File not found: {}!'.format(self.build_info))

        self.json_report = JSONReport(self.build_info)

        if not self.full_build:
            for junit_xml_file in glob.iglob(os.path.join(self.run_reports_dir, '*.xml')):
                self.json_report.updateFromJUnit(junit_xml_file)

    @classmethod
    def fromCMDParser(cls) -> 'ComponentTestBuilder':
        """
        Commandline parser to create instance of this class

        Commandline Arguments
        ---------------------

        --fullBuild : int
            If --fullBuild is 1 all component tests will be built,
            if --fullBuild is 0 (Default) only new components tests will be built.

        --startDir : str
            The search for component starts from this directory (Optional).
            Default is the ComponentSpace root directory.

        --buildFreeCAD: int
            Build FreeCAD Geometry components, default is 0 (False)

        --buildCreo: int
            Build Creo Geometry components, default is 0 (False)

        Returns
        -------
        builder : ComponentTestBuilder
        """

        parser = optparse.OptionParser("usage: %prog [options]")
        parser.add_option("-f", "--fullBuild", dest="full_build",
                          default=0, type="int",
                          help="(re)-build all component tests, default is 0 (False)")
        parser.add_option("-d", "--startDir", dest="start_dir", default=Paths.getComponentSpaceRoot(),
                          type="string", help="The search for components starts from this directory, "
                                              "default is the ComponentSpace root directory.")
        parser.add_option("--buildFreeCAD", dest="build_freecad", default=0,
                          type="int", help="Build FreeCAD Geometry components, default is 0 (False)")
        parser.add_option("--buildCreo", dest="build_creo", default=0,
                          type="int", help="Build Creo Geometry components, default is 0 (False)")

        args = CMDParser.jepSysArgs()
        (options, args) = parser.parse_args(args)
        if len(args) != 0:
            parser.error("incorrect number of arguments")

        full_build = bool(options.full_build)
        start_dir = CMDParser.jepArgPath(options.start_dir)
        build_freecad = bool(options.build_freecad)
        build_creo = bool(options.build_creo)

        print('FullBuild: {}'.format(full_build))
        print('Start Path: {}'.format(start_dir))
        print('Build FreeCAD Tests: {}'.format(build_freecad))
        print('Build Creo Tests: {}'.format(build_creo))

        builder = cls(start_dir, full_build)
        builder.build_freecad = build_freecad
        builder.build_creo = build_creo
        return builder

    def startBuild(self):
        """
        Try to build test projects for all components

        This method could be killed by an external process,
        hence the workaround make sure the intermediate process is stored on the file system.

        """
        if self.full_build:
            print('\nStarting full build of component tests...\n')
        else:
            print('\nStarting build of tests for new components only...\n')

        print('Start Directory: {}'.format(self.start_path))

        components_list = list(self.json_report.getBuildTestComponents(self.full_build))
        self.buildTests(components_list)

    def buildTests(self, components_list: list):
        """
        Try to build tests for all components in the components_list

        Parameters
        ----------
        components_list : list
            List of components to build tests for

        """
        n_tests = len(components_list)

        print('\nBuilding {} component tests'.format(n_tests))

        self.junit_build.startTimer()

        xmp = ExternalMultiprocessing(max_cpu_percent=75, max_ram_percent=75, timeout=self.timeout)

        disp_paths = []
        for index, path_comp in enumerate(components_list):

            disp_path = path_comp.replace(Paths.getComponentSpaceRoot(), '.')
            disp_paths.append(disp_path)

            head, name = os.path.split(path_comp)
            comp_file = os.path.join(head, name, name + '.mop')
            build_tests_py = os.path.join(path_comp, 'tests', 'build_tests.py')
            timestamp = str(datetime.datetime.now().strftime('%H:%M:%S'))
            print('{} Build Test {}/{} ==> {}'.format(timestamp, index + 1, n_tests, disp_path))

            os.makedirs(os.path.join(path_comp, 'tests'), exist_ok=True)

            if os.path.isfile(build_tests_py):
                script_py = build_tests_py
            else:
                script_py = build_external_component_test.__file__

            cmd_args = [Paths.getCSyMSpaceExe(), script_py, comp_file, int(self.build_freecad), int(self.build_creo)]
            xmp.startProcess(cmd_args, wait_to_start=False, wait_to_finish=False)
            xmp.wait()

        '''wait until all processes are done'''
        xmp.waitForCompletion()

        xmp.refresh()
        err_cnt = dict(n_timeout=0, n_errors=0)

        print('\n___Summary___')
        for index, process in enumerate(xmp.processes):
            path_comp = components_list[index]
            status, msg, runTime = xmp.analyzeProcessStatus(process, err_cnt)

            '''set the flag to false so the successful test will not be built next time'''
            build_flag = False if status == 'Successful' else True
            self.json_report.addBuildTests(path_comp, build_flag)
            self.junit_build.addTestCase(path_comp, runTime, msg)

            print('Test {}/{} ==> {} ==> {}'.format(index + 1, n_tests, disp_paths[index], status))

        print('Total Timeout: {}/{}'.format(err_cnt['n_timeout'], n_tests))
        print('Total Errors: {}/{}'.format(err_cnt['n_errors'], n_tests))
        print('Total Failed: {}/{}'.format(err_cnt['n_timeout'] + err_cnt['n_errors'], n_tests))

        '''save at the end of the loop'''
        self.json_report.writeJSON()
        self.junit_build.writeXML()


if __name__ == '__main__':
    '''Quickstart for Debugging / Developing'''
    sp = os.path.join(Paths.getComponentSpaceRoot(), 'Simulation', 'Motor', 'Magnetic', 'PMSM_RBF')
    os.chdir(sp)
    test_builder = ComponentTestBuilder(sp, True)
    test_builder.startBuild()
    test_builder.deployReports()
