# !/usr/bin/python
# -*- coding: <UTF-8> -*-

"""
Python Version : 3.6

$LastChangedRevision: 4767 $
$LastChangedDate:: 2021-09-07 11:29:07 +0200#$

This file is part of SyMSpace.

Copyright (c) 2021, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
All rights reserved.
"""
import json
import argparse
from symspace.osutils.pathutils import CMDParser
from symspace.wrapper.symwrapper import SymProject, SymContainer, SymField, SymParameter


class ProjectCheck:

    def __init__(self, mop_file: str, template_param_dict: dict):
        self.__mop_file = mop_file
        self.__prj = SymProject.load(mop_file)
        self.__template_param_dict = template_param_dict
        self.__save = False

    def runLoop(self, dry_run: bool = False, check_template_description: bool = False,
                check_template_comment: bool = False):
        if self.__prj.getIdentifier() == 'Component':
            for key in self.__template_param_dict.keys():
                cur_param = self.__prj.getObject(key)
                if cur_param is None:
                    print('\tMissing Parameter: {}'.format(key))
                    # raise Exception('Missing Parameter: {}'.format(key))

                    self.addDisplayName(key)
                    self.addOwnerDomain(key, domain='lcm.at')
                    self.addContainer(key, 'Component', 'Component.Packages')
                    self.addContainer(key, 'Component.Packages', 'Component.Packages.Scripts')
                    self.addContainer(key, 'Component', 'Component.Testparameter')
                    self.addField(key, 'Component.Filter', 'Component.Filter.SimulationTool')

                else:
                    if check_template_description:
                        self.checkDescription(cur_param, key)

                    if check_template_comment:
                        self.checkComment(cur_param, key)

            if self.__save and not dry_run:
                self.__prj.save()
                print('\tSaved: {}'.format(self.__mop_file))
        else:
            raise Exception('Missing Parameter: {}'.format('Component'))

        self.__prj.close()

    def checkDescription(self, cur_param: SymParameter, key: str):
        description = self.__template_param_dict[key]['Description']
        cur_description = cur_param.getDescription()
        if description != cur_description:
            # print('\tDescription does not match: {}; {} != {}'.format(key, description, cur_description))
            cur_param.setDescription(description)
            self.__save = True
            print('\tSet description from template: {}'.format(key))

    def checkComment(self, cur_param: SymParameter, key: str):
        comment = self.__template_param_dict[key]['Comment']
        cur_comment = cur_param.getDescription()

        if comment != cur_comment:
            # print('\tComment does not match: {}; {} != {}'.format(key, comment, cur_comment))
            cur_param.setComment(comment)
            self.__save = True
            print('\tSet comment from template: {}'.format(key))

    def addDisplayName(self, key: str):
        if key == 'Component.Info.DisplayName':
            param = self.__prj.getObject('Component.Info.Name')
            if param is not None:
                '''create DisplayName and set it to the same value as Name'''
                container = param.getParent()
                data = param.getData()
                description = self.__template_param_dict[key]['Description']
                comment = self.__template_param_dict[key]['Comment']
                new_param = SymField('DisplayName', description)
                container.addParam(new_param)
                new_param.setComment(comment)
                new_param.setData(data)
                self.__save = True
                print('\tCreated: Component.Info.DisplayName = {}'.format(data))

    def addFunctionName(self, key: str):
        if key == 'Component.Info.FunctionName':
            param = self.__prj.getObject('Component.Info.Name')
            if param is not None:
                '''create FunctionName and set it to the same value as Name'''
                container = param.getParent()
                data = param.getData()
                description = self.__template_param_dict[key]['Description']
                comment = self.__template_param_dict[key]['Comment']
                new_param = SymField('FunctionName', description)
                container.addParam(new_param)
                new_param.setComment(comment)
                new_param.setData(data)
                self.__save = True
                print('\tCreated: Component.Info.FunctionName = {}'.format(data))

    def addOwnerDomain(self, key: str, domain: str):
        if key == 'Component.Info.OwnerDomain':
            param = self.__prj.getObject('Component.Info.Author')
            if param is not None:
                '''create OwnerDomain and set it to the same value as the Author Domain'''
                author = param.getData()
                if author.endswith(domain):
                    container = param.getParent()
                    description = self.__template_param_dict[key]['Description']
                    comment = self.__template_param_dict[key]['Comment']
                    new_param = SymField('DisplayName', description)
                    container.addParam(new_param)
                    new_param.setComment(comment)
                    new_param.setData(domain)
                    self.__save = True
                    print('\tCreated: Component.Info.OwnerDomain = {}'.format(domain))

    def addField(self, key: str, parent_id: str, field_id: str):
        if key == field_id:
            '''Create the missing field'''
            container = self.__prj.getObject(parent_id)
            if isinstance(container, SymContainer):
                name = field_id.split('.')[-1]
                description = self.__template_param_dict[key]['Description']
                comment = self.__template_param_dict[key]['Comment']
                new_param = SymField(name)
                container.addParam(new_param)
                new_param.setDescription(description)
                new_param.setComment(comment)
                self.__save = True
                print('\tCreated: {}'.format(field_id))

    def addContainer(self, key: str, parent_id: str, container_id: str):
        if key == container_id:
            '''Create the missing container'''
            container = self.__prj.getObject(parent_id)
            if isinstance(container, SymContainer):
                name = container_id.split('.')[-1]
                description = self.__template_param_dict[key]['Description']
                comment = self.__template_param_dict[key]['Comment']
                new_param = SymContainer(name)
                container.addParam(new_param)
                new_param.setDescription(description)
                new_param.setComment(comment)
                self.__save = True
                print('\tCreated: {}'.format(container_id))


def main():
    parser = argparse.ArgumentParser(description='Execute build script and save error log')
    parser.add_argument('mop_file', help='SyMSpace Project file to process')
    parser.add_argument('json_file', help='JSON file with template structure')
    parser.add_argument('dry_run', help='Do not save changes', type=int)
    parser.add_argument('check_comment', help='Check if parameter comments are same as in the template', type=int)
    parser.add_argument('check_description', help='Check if parameter description are same as in the template',
                        type=int)

    args = CMDParser.jepSysArgs()
    args = parser.parse_args(args)

    mop_file = CMDParser.jepArgPath(args.mop_file)
    json_file = CMDParser.jepArgPath(args.json_file)
    dry_run = bool(args.dry_run)
    check_comment = bool(args.check_comment)
    check_description = bool(args.check_description)

    print('\tMop File: {}'.format(mop_file))
    # print('\tJSON File: {}'.format(json_file))
    # print('\tdry_run: {}'.format(dry_run))
    # print('\tcheck_comment: {}'.format(check_comment))
    # print('\tcheck_description: {}'.format(check_description))

    with open(json_file, 'r') as fh:
        template_param_dict = json.load(fh)

    pc = ProjectCheck(mop_file, template_param_dict)
    pc.runLoop(dry_run=dry_run, check_template_comment=check_comment, check_template_description=check_description)


if __name__ == '__main__':
    main()
